// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "DataStream.h"

//------------------------------------------------------------------------------

using hu::varadiistvan::scpl::io::DataStream;

using std::string;
using std::min;

//------------------------------------------------------------------------------

size_t DataStream::readLength()
{
    size_t length = 0;

    uint8_t d = 0;
    do {
        d = readU8();
        length <<= 7;
        length |= d&0x7f;
    } while( (d&0x80)==0x80 );

    return length;
}

//------------------------------------------------------------------------------

string DataStream::readString()
{
    size_t length = readLength();
    if (!*this) return "";

    char* buffer = new char[length];
    if (read(buffer, length)) {
        string str(buffer, length);
        delete buffer;
        return str;
    } else {
        delete buffer;
        return "";
    }
}

//------------------------------------------------------------------------------

size_t DataStream::readString(char* dest, size_t capacity)
{
    size_t length = readLength();
    size_t destLength = 0;
    if (*this && length>0) {
        size_t destLength = min(length, capacity - 1);
        if (read(dest, destLength)) {
            if (destLength<length) {
                if (!skip(length - destLength)) {
                    length = destLength = 0;
                }
            }
        } else {
            length = destLength = 0;
        }
    } else {
        length = 0;
    }

    dest[destLength] = '\0';

    return length;
}

//------------------------------------------------------------------------------

bool DataStream::writeLength(size_t length)
{
    do {
        uint8_t x = length&0x7f;
        length >>= 7;
        if (length>0) x |= 0x80;
        if (!writeU8(x)) return false;
    } while(length>0);
    return true;
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
