// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

//------------------------------------------------------------------------------

#include "LocalServerSocketBase.h"

#include "LocalSocket.h"

//------------------------------------------------------------------------------

using hu::varadiistvan::scpl::io::LocalServerSocketBase;

//------------------------------------------------------------------------------

bool LocalServerSocketBase::createNamedPipe(const char* path)
{
    handle = CreateNamedPipe(path, PIPE_ACCESS_DUPLEX |
                             ((waiter==0) ? 0 : FILE_FLAG_OVERLAPPED),
                             PIPE_TYPE_BYTE | PIPE_READMODE_BYTE,
                             PIPE_UNLIMITED_INSTANCES,
                             BufferedStream::DEFAULT_CAPACITY,
                             BufferedStream::DEFAULT_CAPACITY,
                             0, 0);
    if (handle==0) {
        setErrorCode(GetLastError());
    }
    return handle!=0;
}

//------------------------------------------------------------------------------

bool LocalServerSocketBase::connectNamedPipe(Overlapped* overlapped)
{
    if (overlapped!=0) overlapped->reset();
    bool result = ConnectNamedPipe(handle,
                                   (overlapped==0) ? 0 : overlapped->get());
    DWORD error = GetLastError();
    if (!result &&
        (overlapped==0 ||
         (error!=ERROR_IO_PENDING && error!=ERROR_IO_INCOMPLETE)))
    {
        setErrorCode(error);
        return false;
    } else if (overlapped==0) {
        return result;
    } else {
        DWORD dummy = 0;
        return overlapped->getResult(dummy, handle);
    }
}

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
